;
; creatures PETSCII
;
; a tribute to Shine/G*P
;
; released by Arkanix Labs in 2020
;
; www.arkanixlabs.com
;
; code and graphics by
; aNdy (CSDb id=6404)
;
; music by
; steve rowlands (CSDb id=4043)
;
; ntsc testing by
; moloch (CSDb id=930)
;
; this source is formatted for the ACME cross assembler
; which can be downloaded from...
; http://sourceforge.net/projects/acme-crossass/
;
; the final build was compressed and squished using
; Exomizer 2 which can be downloaded from...
; http://hem.bredband.net/magli143/exo/
;
; questions about this code or about how the binaries were
; created can be asked visiting the arkanix labs forum...
;
; https://www.arkanixlabs.com/forum/index.php
;

; the name and file format for the assembled program

		!to "sourcebuild.prg",cbm

; claw in the binary data...
		
		* = $2800
		!binary "data/font.chr"
		
		* = $10ff
music	!binary "data\music.prg",,2

; label assignments for keyboard control

cpra				= $dc00				; cia #1, port register a
cprb				= $dc01				; cia #1, port register b

; some more label assignments (variables)

sync				= $50				; interupt sync variable
video_type			= $51				; populated with 0=pal or 1=ntsc
music_tmr			= $52				; timer for music frame skip on ntsc		


; add a BASIC startline (SYS 16384) for auto-run after loading

		* = $0801
		!word code_entry-2
		!byte $00,$00,$9e
		!text "16384"
		!byte $00,$00,$00

; entry point for the code

		* = $4000					; hence sys 16384!
				
code_entry

; detect what machine type is being used...

start_scan
		lda $d012
chkntsc
		cmp $d012
		beq chkntsc
		bmi start_scan
		adc #$00
		lsr
		bcs isntsc
		lsr
		bcs isoldntsc
		ldx #$00					; check drean or pal
		lda #$10
chkpal
		inx
		cmp $d012
		bne chkpal
		cpx #$70
		bcc ispal
isdrean
		lda #$01
		sta video_type				; video = ntsc
		jmp detect_dun
isntsc	
		lda #$01
		sta video_type				; video = ntsc
		jmp detect_dun
isoldntsc
		lda #$01
		sta video_type				; video = ntsc
		jmp detect_dun
ispal
		lda #$00
		sta video_type				; video = pal

detect_dun
		lda #$35					; kill EVERYTHING (MWHAHA!) except $f000-$ffff
		sta $01

; stop interrupts, disable the roms and set up nmi and irq interrupt pointers	

		sei							; disable maskable IRQs		
		
; setup nmi

		lda #<nmi
		sta $fffa
		lda #>nmi
		sta $fffb
		
; setup irq's

		lda #<int
		sta $fffe
		lda #>int
		sta $ffff

		lda #$7f
		sta $dc0d			; disable timer interrupts which can be generated by the two CIA chips
		sta $dd0d			; the kernal uses such an interrupt to flash the cursor and scan the keyboard 
							; so we'd better stop it...
		lda $dc0d			; by reading these two registers we negate any pending CIA irqs.
		lda $dd0d			; if we don't do this, a pending irq might occur after we finish setting up our irq

		lda #$00			; where raster starts on screen, usually at top but could be different
		sta $d012

		lda #$1b			; as there are more than 256 rasterlines, the topmost bit of $d011 serves as
		sta $d011			; the 9th bit for the rasterline we want our irq to be triggered.
		
		lda #$01
		sta $d019			; clear interupt bit
		sta $d01a			; tell vicii to generate interupt

		lda #$35			; turn off basic and kernal rom thus
		sta $01				; c64 now sees RAM everywhere except at $d000-$e000
		
		lda #$03			; select video bank
		sta $dd00		
						
		lda #$1a			; use our custom charset (lowercase rom)
		sta $d018

		cli
		
; prepare the music	

		lda #$01					; creatures 'get ready' tune
		jsr music+$00		
		
; setup the screen for the very first run

		jsr clear_screen			; clear screen with subroutine

; print intro message		
				
		ldx #$00
text_loop
		lda intro_1,x				; pull in the data for the
		sta $0590,x					; 3 line 'intro' text
		lda intro_2,x				; and print it on screen!
		sta $05e0,x		
		lda intro_3,x
		sta $0630,x			
	
		lda #$07					; colour those lines!
		sta $d990,x
		lda #$0a		
		sta $d9e0,x
		lda #$08	
		sta $da30,x		

		inx
		cpx #$28
		bne text_loop

; give time to read the intro	
						
		ldy #$ff					; wait a bit...
		jsr sync_wait_long	

		jsr clear_screen			; clear screen again
		
; print up the main screen from screen ram table			
		
		ldx #$00					; zero x register
scrnram_loop						; load up whatver is in the screen
		lda screen_ram,x			; ram table and write to the
		sta $0400,x					; screen ram
		lda screen_ram+250,x
		sta $0400+250,x
		lda screen_ram+500,x
		sta $0400+500,x
		lda screen_ram+750,x
		sta $0400+750,x	
		inx
		cpx #$fa					; done 250 writes?
		bne scrnram_loop			; all written? no? loop back!
		
; print the compressed main screen colour data table to colour ram
			
		ldx #$00					; zero x register
		ldy #$00					; zero y register
colram_loop
		lda colour_ram,x			; load up a byte
		sta $d800+1,y				; store a nybble to screen
		lsr
		lsr
		lsr
		lsr
		sta $d800,y					; store the other nybble

		lda colour_ram+125,x
		sta $d800+251,y
		lsr
		lsr
		lsr
		lsr
		sta $d800+250,y

		lda colour_ram+250,x
		sta $d800+501,y
		lsr
		lsr
		lsr
		lsr
		sta $d800+500,y

		lda colour_ram+375,x
		sta $d800+751,y
		lsr
		lsr
		lsr
		lsr
		sta $d800+750,y

		iny
		iny
		inx
		cpx #$7d
		bne colram_loop
		
; main music track
		
		lda #$02					; creatures torture screen 2 music
		jsr music+$00			
		
; main loop to sync irq and detect space keypress

mainloop

		jsr sync_wait				; wait until irq finishes!

		lda #$7f 					; load $dc00...
		sta cpra					; ...with bit row 7
		
		lda cprb
		cmp #$ef					; check space press
		bne kcheck_dun
		
		jsr key_debounce
		jmp exit_demo
		
kcheck_dun			
		jmp mainloop
	
; space has been pressed so reset some registers

exit_demo
		sei
		lda #$37
		sta $01

		lda #$00
		sta $d011
		sta $d020
		sta $d021
		sta $d418

; reset the C64

		jmp $fce2			
												

; irq interrupt =================================

int
		pha  
		txa
		pha
		tya
		pha

		lda $d019
		and #$01
		sta $d019

; play some music!

		lda video_type				; what machine is being used
		cmp #$00					; PAL?
		beq call_music_driver		; yes? play the music!
									; no, it's ntsc?
		ldx music_tmr				; load music timer
		inx							; increase it
		cpx #$06					; is it 6?
		bne	music_play				; no? play music!
									; yes?
		ldx #$00					; reset the timer
		stx music_tmr				; then skip playing
									; the music on this occasion
		jmp set_sync				; to slow it for ntsc!

music_play
		stx	music_tmr
		
call_music_driver
		jsr music+$04

; set sync variable for main line code synchronisation

set_sync
		lda #$01					; set the sync flag to '1'		
		sta sync					; so the main code runs!
		
		pla
		tay
		pla
		tax
		pla
nmi		rti

; end irq interrupt =============================


; subroutines ===================================


; clear the screen!

clear_screen
        lda #$00					; colour black to...
		sta $d020					; border and
		sta $d021					; background
		tax							; shove 0 into x register
		lda #$20					; load 'space' into accum
clsloop
		sta $0400,x					; wipe the screen
		sta $0500,x					; with #$20 (space character)
		sta $0600,x
		sta $0700,x
		dex
		bne clsloop

		rts

; debounce subroutines for keyboard -------------------

key_debounce		
		lda cprb						; is there a keypress still?
		cmp #$ff						; an idiot is still pressing something???
		bne key_debounce				; lock them in a loop till they stop!!
		
		rts	

; irq sync wait subroutine ---------------------------------- 
	   	 
sync_wait								; wait for sync update in irq code!
		lda #$00						; waits until the sync flag is
		sta sync						; set at the end of the interupt
sw_loop									; code further down.
		cmp sync						; no code in the main loop above
		beq sw_loop						; is executed until the interupt
										; has run it's course!
		rts

; a sync wait delay subroutine ---------------------------------- 		
		
sync_wait_long						
		jsr sync_wait
		dey
		bne sync_wait_long

		rts							
			   	
; end subroutines ===============================


; screen data here! ==========================


; text data for intro message		
		
intro_1		!scr "Arkanix Labs presents  Creatures PETSCII"
intro_2		!scr "  Another small tribute to Shine / G*P  "
intro_3		!scr "Code/GFX: aNdy     Music: Steve Rowlands"


; data below generated/exported with screen editor 'ChillED'
; from Arkanix Labs (release coming soon!)-

; main screen ram data

screen_ram
!byte $AA,$E9,$A3,$AA,$25,$66,$E6,$5C,$66,$E6,$5F,$E9,$25,$A3,$AA,$66,$66,$5E,$5C,$20,$C7,$FC,$20,$25,$E9,$A3,$5F,$E6,$E6,$66,$E9,$E9,$AA,$E9,$A5,$A3,$5F,$25,$69,$AA
!byte $E9,$25,$E6,$66,$5C,$5C,$E6,$66,$20,$68,$66,$AA,$E9,$25,$69,$5C,$20,$5C,$20,$FE,$E6,$E8,$FC,$7C,$E6,$66,$E6,$66,$E6,$66,$5F,$A3,$E9,$25,$A3,$E9,$AB,$66,$AA,$69
!byte $25,$5C,$5C,$25,$5C,$5C,$5C,$20,$20,$5E,$5E,$5F,$A3,$69,$5C,$20,$20,$20,$67,$ED,$EE,$66,$E6,$5C,$22,$66,$E6,$23,$66,$E6,$66,$5F,$A5,$E9,$E9,$66,$66,$E6,$E6,$A3
!byte $E6,$5C,$5C,$E6,$20,$5C,$20,$20,$25,$66,$EC,$22,$66,$5E,$66,$20,$20,$20,$76,$66,$ED,$E8,$E6,$FC,$20,$22,$66,$E6,$E6,$23,$66,$66,$5F,$A5,$66,$E6,$E6,$23,$66,$25
!byte $5C,$5C,$5C,$5C,$20,$5C,$20,$20,$66,$5E,$20,$20,$22,$25,$5E,$5C,$20,$20,$67,$7F,$E6,$66,$66,$E6,$FC,$20,$22,$5C,$66,$22,$5C,$E6,$25,$28,$20,$5C,$25,$22,$20,$E6
!byte $20,$20,$5C,$20,$20,$5C,$20,$68,$E6,$66,$68,$E6,$68,$66,$E6,$68,$68,$E6,$68,$9B,$E8,$E8,$9D,$68,$22,$20,$20,$20,$20,$20,$20,$29,$25,$5C,$20,$20,$20,$20,$20,$20
!byte $68,$68,$68,$68,$20,$5C,$E1,$A3,$E9,$E6,$A5,$E6,$AA,$A3,$A5,$A3,$AA,$A3,$A5,$A3,$A5,$A5,$FB,$7E,$20,$20,$7B,$6C,$20,$20,$20,$20,$E6,$E6,$20,$FF,$E8,$7F,$20,$20
!byte $E9,$66,$E6,$66,$20,$5C,$7C,$DF,$A3,$AA,$A3,$A5,$A3,$F6,$5C,$A3,$FB,$A3,$7F,$A5,$EC,$7C,$20,$20,$20,$FE,$85,$85,$FC,$20,$20,$20,$29,$66,$20,$E1,$E8,$5C,$20,$20
!byte $23,$E6,$66,$25,$20,$5C,$20,$E1,$A5,$A3,$E9,$25,$7E,$22,$8F,$68,$68,$68,$68,$68,$68,$68,$68,$68,$68,$E1,$ED,$FD,$61,$68,$68,$68,$66,$E6,$68,$68,$E8,$5C,$20,$20
!byte $25,$5C,$5C,$22,$20,$5C,$20,$7C,$DF,$E9,$25,$22,$20,$20,$5C,$20,$68,$68,$E6,$E6,$68,$68,$E6,$E6,$68,$68,$F2,$F2,$68,$E6,$68,$E6,$E6,$5F,$68,$E6,$E8,$66,$68,$20
!byte $27,$20,$5C,$20,$20,$5C,$20,$20,$7F,$A3,$FF,$20,$20,$20,$5C,$20,$E6,$5F,$E9,$E6,$A5,$E6,$AA,$A3,$A5,$A3,$AA,$A3,$A5,$A5,$A3,$AA,$A3,$A5,$A3,$A5,$A5,$E6,$E6,$20
!byte $2E,$20,$5C,$20,$20,$5C,$20,$20,$7C,$21,$7E,$20,$20,$FE,$FC,$20,$20,$22,$A3,$AA,$A3,$25,$7E,$7C,$25,$25,$FB,$A3,$7F,$23,$25,$22,$20,$20,$22,$22,$E6,$5C,$22,$20
!byte $66,$20,$5C,$20,$20,$5C,$20,$20,$20,$2A,$20,$20,$FE,$A0,$A0,$FC,$20,$20,$20,$64,$64,$64,$64,$64,$20,$27,$20,$20,$22,$20,$20,$20,$20,$7B,$6C,$F8,$62,$20,$20,$20
!byte $27,$20,$5C,$20,$20,$5C,$20,$20,$20,$20,$20,$6C,$A0,$B1,$B0,$A0,$7B,$20,$20,$C8,$C5,$CC,$D0,$A1,$7B,$0F,$E8,$FC,$20,$20,$20,$20,$7F,$A7,$DC,$A0,$A2,$5F,$7B,$20
!byte $20,$20,$5C,$20,$20,$5C,$20,$20,$20,$20,$20,$E1,$D4,$CF,$CE,$D3,$61,$20,$20,$20,$20,$68,$68,$68,$68,$EC,$7F,$7F,$68,$68,$68,$7F,$A7,$EC,$DC,$A0,$BD,$A0,$A2,$F8
!byte $20,$20,$5C,$20,$20,$5C,$20,$20,$20,$20,$20,$7C,$E2,$E2,$E2,$E2,$7E,$20,$20,$20,$FE,$8F,$8F,$8F,$8F,$8F,$8F,$8F,$8F,$8F,$8F,$8F,$8F,$FE,$C1,$C3,$CD,$C5,$BE,$BE
!byte $20,$5A,$5C,$20,$20,$5C,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$20,$7C,$22,$22,$22,$FE,$A2,$A2,$A2,$A2,$A2,$8F,$A2,$A2,$A7,$A0,$A2,$A2,$A0,$E8,$E8
!byte $1A,$20,$5C,$20,$20,$5C,$20,$20,$20,$20,$20,$20,$20,$2C,$3B,$20,$20,$20,$20,$2E,$2E,$2C,$20,$20,$FB,$E8,$E8,$FC,$E8,$A0,$DD,$D3,$C1,$D7,$CD,$C9,$CC,$CC,$7E,$20
!byte $3B,$6C,$A5,$20,$20,$5C,$20,$20,$20,$20,$20,$20,$6A,$8F,$8F,$FC,$20,$20,$E9,$7F,$E6,$FF,$DF,$20,$20,$DC,$DC,$20,$7C,$7F,$E8,$E8,$E8,$E8,$EC,$E8,$E8,$7E,$20,$20
!byte $AE,$5C,$5C,$20,$20,$5C,$20,$20,$20,$20,$20,$20,$6C,$7F,$E8,$DF,$FC,$20,$FB,$EC,$6F,$FB,$EC,$20,$20,$DC,$DC,$20,$20,$68,$25,$20,$DC,$DC,$7C,$FC,$FE,$7E,$20,$20
!byte $E6,$66,$C5,$20,$20,$5C,$20,$20,$76,$20,$75,$20,$B3,$EC,$28,$7F,$29,$78,$77,$FE,$8F,$FC,$68,$25,$25,$DC,$DC,$25,$25,$25,$27,$20,$DC,$DC,$E1,$B0,$B0,$61,$8F,$20
!byte $EE,$F0,$7F,$68,$DF,$E6,$E9,$68,$76,$F7,$75,$68,$FE,$68,$FE,$22,$FC,$68,$FE,$BA,$BD,$BA,$FC,$68,$2C,$DC,$DC,$68,$2E,$68,$2C,$68,$DC,$DC,$FF,$ED,$FD,$7F,$FC,$68
!byte $E6,$66,$E6,$66,$E6,$66,$E6,$66,$E6,$66,$E6,$66,$E6,$66,$E6,$66,$E6,$66,$E6,$66,$E6,$66,$E6,$66,$E6,$66,$E6,$66,$E6,$66,$E6,$66,$E6,$66,$E1,$EE,$F0,$61,$E6,$66
!byte $E6,$7C,$FC,$FE,$7E,$AA,$A3,$A5,$A5,$A3,$AA,$A3,$A5,$A3,$E6,$AA,$A3,$A5,$A3,$AA,$A3,$A5,$A5,$E6,$AA,$A3,$A5,$A3,$AA,$A3,$A5,$A5,$A3,$AA,$A3,$A5,$A3,$A5,$A5,$E6
!byte $66,$E1,$B0,$B0,$61,$38,$20,$20,$20,$53,$43,$4F,$52,$45,$20,$6D,$73,$28,$29,$28,$29,$28,$29,$20,$25,$22,$20,$01,$4E,$04,$19,$20,$25,$22,$20,$30,$32,$20,$22,$25

; main colour ram data

colour_ram
!byte $5D,$55,$55,$89,$98,$5D,$55,$58,$99,$90,$7B,$0D,$D5,$58,$85,$5D,$5D,$55,$55,$D5
!byte $D5,$58,$89,$99,$09,$85,$D5,$89,$09,$05,$DD,$B8,$88,$89,$99,$55,$D5,$55,$98,$5D
!byte $58,$99,$99,$90,$09,$85,$58,$90,$00,$BD,$C7,$CB,$99,$99,$99,$85,$5D,$58,$99,$85
!byte $85,$99,$99,$00,$99,$99,$99,$90,$00,$B7,$C7,$5B,$09,$99,$99,$98,$55,$89,$99,$95
!byte $59,$99,$09,$00,$99,$09,$99,$99,$99,$B5,$7D,$5C,$B0,$99,$99,$99,$55,$09,$99,$09
!byte $99,$90,$09,$05,$55,$55,$55,$55,$55,$5B,$55,$B5,$B0,$00,$00,$09,$99,$00,$00,$00
!byte $55,$55,$09,$9A,$77,$A7,$AA,$8A,$A8,$A8,$89,$99,$00,$BB,$00,$00,$89,$08,$89,$00
!byte $8A,$89,$09,$98,$8A,$88,$89,$88,$99,$99,$99,$90,$0B,$77,$B0,$00,$A9,$08,$99,$00
!byte $98,$A9,$09,$09,$A8,$89,$99,$79,$99,$99,$99,$99,$9B,$FC,$B9,$99,$89,$99,$99,$00
!byte $99,$99,$09,$08,$A8,$99,$00,$90,$55,$55,$55,$55,$55,$BB,$55,$55,$89,$59,$99,$50
!byte $90,$90,$09,$00,$89,$90,$00,$90,$58,$77,$A7,$AA,$8A,$A8,$A8,$AA,$8A,$88,$99,$50
!byte $90,$90,$09,$00,$94,$90,$0C,$B0,$09,$8A,$89,$99,$98,$99,$99,$89,$88,$99,$99,$90
!byte $90,$90,$09,$00,$0E,$00,$CF,$FB,$00,$01,$11,$11,$99,$00,$90,$00,$0C,$F1,$F0,$00
!byte $90,$90,$09,$00,$00,$0C,$FF,$FF,$B0,$01,$11,$11,$17,$BB,$00,$00,$CF,$11,$1F,$F0
!byte $00,$90,$09,$00,$00,$0C,$FF,$FF,$B0,$00,$0F,$CF,$CB,$AA,$CF,$CC,$FF,$11,$F1,$11
!byte $00,$90,$09,$00,$00,$0C,$BB,$BB,$B0,$00,$C6,$66,$66,$66,$66,$66,$61,$11,$11,$11
!byte $01,$90,$09,$00,$00,$00,$00,$00,$00,$00,$BC,$FC,$F1,$11,$11,$21,$11,$11,$11,$FC
!byte $F0,$90,$09,$00,$00,$00,$04,$40,$00,$07,$77,$00,$F3,$3F,$11,$11,$11,$11,$11,$BB
!byte $49,$10,$09,$00,$00,$00,$97,$79,$00,$17,$77,$10,$0F,$9B,$BC,$FF,$33,$FF,$CC,$00
!byte $99,$90,$09,$00,$00,$00,$44,$48,$90,$FC,$FF,$C0,$01,$80,$02,$20,$F9,$AB,$BA,$00
!byte $E4,$10,$09,$00,$BE,$B0,$19,$F4,$F9,$9F,$8C,$22,$21,$82,$22,$20,$18,$B7,$7B,$20
!byte $49,$99,$78,$99,$BE,$B9,$98,$99,$99,$F1,$1F,$C9,$9C,$99,$99,$99,$C9,$BA,$CB,$79
!byte $55,$55,$55,$55,$55,$55,$55,$55,$55,$55,$55,$55,$55,$55,$55,$55,$55,$BA,$AB,$55
!byte $7A,$BB,$AA,$8A,$8A,$A8,$A8,$7A,$A8,$AA,$8A,$87,$AA,$8A,$A8,$A8,$AA,$8A,$88,$99
!byte $9B,$77,$B1,$00,$01,$11,$11,$01,$11,$11,$11,$10,$99,$0A,$AA,$A0,$99,$01,$10,$99

